@@ -27,19 +27,15 @@ from mo_future import (
     start_new_thread,
     text,
     decorate,
-    PY3,
 )
 from mo_logs import Except, Log
-from mo_threads.profiles import CProfiler, write_profiles
 from mo_threads.signals import AndSignals, Signal
 from mo_threads.till import Till
 
 DEBUG = False
 
-PLEASE_STOP = str("please_stop")  # REQUIRED thread PARAMETER TO SIGNAL STOP
-PARENT_THREAD = str(
-    "parent_thread"
-)  # OPTIONAL PARAMETER TO ASSIGN THREAD TO SOMETHING OTHER THAN CURRENT THREAD
+PLEASE_STOP = "please_stop"  # REQUIRED thread PARAMETER TO SIGNAL STOP
+PARENT_THREAD = "parent_thread" # OPTIONAL PARAMETER TO ASSIGN THREAD TO SOMETHING OTHER THAN CURRENT THREAD
 MAX_DATETIME = datetime(2286, 11, 20, 17, 46, 39)
 DEFAULT_WAIT_TIME = timedelta(minutes=10)
 THREAD_STOP = "stop"
@@ -47,10 +43,13 @@ THREAD_TIMEOUT = "TIMEOUT"
 
 datetime.strptime("2012-01-01", "%Y-%m-%d")  # http://bugs.python.org/issue7980
 
-if PY3:
+cprofiler_stats = None  # ACCUMULATION OF STATS FROM ALL THREADS
+
+
+try:
     STDOUT = sys.stdout.buffer
     STDIN = sys.stdin.buffer
-else:
+except Exception:
     STDOUT = sys.stdout
     STDIN = sys.stdin
 
@@ -176,7 +175,9 @@ class MainThread(BaseThread):
             self.timers.stop()
             self.timers.join()
 
-            write_profiles(self.cprofiler)
+            if cprofiler_stats is not None:
+                from mo_threads.profiles import write_profiles
+                write_profiles(self.cprofiler)
             DEBUG and Log.note("Thread {{name|quote}} now stopped", name=self.name)
             self.stopped.go()
 
@@ -301,6 +302,7 @@ class Thread(BaseThread):
         DEBUG and Log.note("Thread {{name|quote}} got request to stop", name=self.name)
 
     def _run(self):
+        self.please_stop.remove_go(self.start)
         self.id = get_ident()
         with RegisterThread(self):
             try:
@@ -373,11 +375,11 @@ class Thread(BaseThread):
                             # THREAD FAILURES ARE A PROBLEM ONLY IF NO ONE WILL BE JOINING WITH IT
                             try:
                                 Log.error(
-                                    "Problem in thread {{name|quote}}", name=self.name, cause=e
+                                    "Problem in thread {{name|quote}}", name=self.name, cause=self.end_of_thread.exception
                                 )
-                            except Exception:
+                            except Exception as cause:
                                 sys.stderr.write(
-                                    str("ERROR in thread: " + self.name + " " + text(e) + "\n")
+                                    str("ERROR in thread: " + self.name + " " + text(cause) + "\n")
                                 )
                         elif self.end_of_thread.response != None:
                             Log.warning(
@@ -458,8 +460,10 @@ class Thread(BaseThread):
 
         if output is None:
             thread = BaseThread(ident)
-            thread.cprofiler = CProfiler()
-            thread.cprofiler.__enter__()
+            if cprofiler_stats is not None:
+                from mo_threads.profiles import CProfiler
+                thread.cprofiler = CProfiler()
+                thread.cprofiler.__enter__()
             with ALL_LOCK:
                 ALL[ident] = thread
             Log.warning(
@@ -468,6 +472,11 @@ class Thread(BaseThread):
             return thread
         return output
 
+    @staticmethod
+    def join_all(threads):
+        for t in threads:
+            t.join()
+
 
 class RegisterThread(object):
     """
@@ -486,12 +495,19 @@ class RegisterThread(object):
     def __enter__(self):
         with ALL_LOCK:
             ALL[self.thread.id] = self.thread
-        self.thread.cprofiler = CProfiler()
-        self.thread.cprofiler.__enter__()
+        if cprofiler_stats is not None:
+            from mo_threads.profiles import CProfiler
+            cprofiler = self.thread.cprofiler = CProfiler()
+            cprofiler.__enter__()
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        self.thread.cprofiler.__exit__(exc_type, exc_val, exc_tb)
+        # PYTHON WILL REMOVE GLOBAL VAR BEFORE END-OF-THREAD
+        all_lock = ALL_LOCK
+        all = ALL
+
+        if cprofiler_stats is not None:
+            self.thread.cprofiler.__exit__(exc_type, exc_val, exc_tb)
         with self.thread.child_locker:
             if self.thread.children:
                 Log.error(
@@ -499,8 +515,8 @@ class RegisterThread(object):
                     children=[c.name for c in self.thread.children],
                     thread=self.thread.name
                 )
-        with ALL_LOCK:
-            del ALL[self.thread.id]
+        with all_lock:
+            del all[self.thread.id]
 
 
 def register_thread(func):
